
/* Chrysalide - Outil d'analyse de fichiers binaires
 * reglist.c - accès à la mémorie à partir d'un registre et d'un décallage
 *
 * Copyright (C) 2014-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "reglist.h"


#include <assert.h>
#include <malloc.h>


#include "../../../operand-int.h"
#include "../../../register.h"



/* Définition d'un opérande listant une série de registres ARM (instance) */
struct _GArmV7RegListOperand
{
    GArchOperand parent;                    /* Instance parente            */

    GArmV7Register **registers;             /* Liste de registres intégrés */
    size_t count;                           /* Taille de cette liste       */

};


/* Définition d'un opérande listant une série de registres ARM (classe) */
struct _GArmV7RegListOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

};


/* Initialise la classe des listes de registres ARM. */
static void g_armv7_reglist_operand_class_init(GArmV7RegListOperandClass *);

/* Initialise une instance de liste de registres ARM. */
static void g_armv7_reglist_operand_init(GArmV7RegListOperand *);

/* Supprime toutes les références externes. */
static void g_armv7_reglist_operand_dispose(GArmV7RegListOperand *);

/* Procède à la libération totale de la mémoire. */
static void g_armv7_reglist_operand_finalize(GArmV7RegListOperand *);

/* Traduit un opérande en version humainement lisible. */
static void g_armv7_reglist_operand_print(const GArmV7RegListOperand *, GBufferLine *, AsmSyntax);



/* Indique le type défini par la GLib pour une liste de registres ARM. */
G_DEFINE_TYPE(GArmV7RegListOperand, g_armv7_reglist_operand, G_TYPE_ARCH_OPERAND);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des listes de registres ARM.            *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_reglist_operand_class_init(GArmV7RegListOperandClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GArchOperandClass *operand;             /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);
    operand = G_ARCH_OPERAND_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_armv7_reglist_operand_dispose;
    object->finalize = (GObjectFinalizeFunc)g_armv7_reglist_operand_finalize;

    operand->print = (operand_print_fc)g_armv7_reglist_operand_print;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise une instance de liste de registres ARM.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_reglist_operand_init(GArmV7RegListOperand *operand)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_reglist_operand_dispose(GArmV7RegListOperand *operand)
{
    size_t i;                               /* Boucle de parcours          */

    for (i = 0; i < operand->count; i++)
        g_object_unref(G_OBJECT(operand->registers[i]));

    G_OBJECT_CLASS(g_armv7_reglist_operand_parent_class)->dispose(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_reglist_operand_finalize(GArmV7RegListOperand *operand)
{
    if (operand->registers != NULL)
        free(operand->registers);

    G_OBJECT_CLASS(g_armv7_reglist_operand_parent_class)->finalize(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Crée une liste vierge de registres ARM.                      *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_armv7_reglist_operand_new(void)
{
    GArmV7RegListOperand *result;             /* Structure à retourner       */

    result = g_object_new(G_TYPE_ARMV7_REGLIST_OPERAND, NULL);

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                line    = ligne tampon où imprimer l'opérande donné.         *
*                syntax  = type de représentation demandée.                   *
*                                                                             *
*  Description : Traduit un opérande en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_reglist_operand_print(const GArmV7RegListOperand *operand, GBufferLine *line, AsmSyntax syntax)
{
    size_t i;                               /* Boucle de parcours          */

    g_buffer_line_append_text(line, BLC_ASSEMBLY, "{", 1, RTT_HOOK, NULL);

    for (i = 0; i < operand->count; i++)
    {
        if (i > 0)
        {
            g_buffer_line_append_text(line, BLC_ASSEMBLY, ",", 1, RTT_PUNCT, NULL);
            g_buffer_line_append_text(line, BLC_ASSEMBLY, " ", 1, RTT_RAW, NULL);
        }

        g_arch_register_print(G_ARCH_REGISTER(operand->registers[i]), line, syntax);

    }

    g_buffer_line_append_text(line, BLC_ASSEMBLY, "}", 1, RTT_HOOK, NULL);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand  = liste de registres à compléter.                   *
*                selected = masque de bits pour les registres à intégrer.     *
*                                                                             *
*  Description : Remplit une liste de registres de registres ARM.             *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_armv7_reglist_load_registers(GArmV7RegListOperand *operand, uint32_t selected)
{
    uint8_t i;                              /* Boucle de parcours          */
    GArmV7Register *reg;                    /* Nouveau registre à intégrer */

    for (i = 18; i < 32; i++)
        if (selected & (1 << i)) return false;

    for (i = 0; i < 18; i++)
    {
        if ((selected & (1 << i)) == 0) continue;

        reg = g_armv7_register_new(i);
        g_armv7_reglist_add_register(operand, reg);

    }

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = liste de registres à compléter.                    *
*                reg     = nouveau registre à intégrer.                       *
*                                                                             *
*  Description : Ajoute un registre à une liste de registres ARM.             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_armv7_reglist_add_register(GArmV7RegListOperand *operand, GArmV7Register *reg)
{
    operand->registers = (GArmV7Register **)realloc(operand->registers,
                                                    ++operand->count * sizeof(GArmV7Register *));

    operand->registers[operand->count - 1] = reg;

}



/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à consulter.                              *
*                                                                             *
*  Description : Compte le nombre de registres ARM composant la liste.        *
*                                                                             *
*  Retour      : Nombre positif ou nul.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

size_t g_armv7_reglist_count_registers(const GArmV7RegListOperand *operand)
{
    return operand->count;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à consulter.                              *
*                index   = indice de l'élément à fournier.                    *
*                                                                             *
*  Description : Founit un élément donné d'une liste de registres ARM.        *
*                                                                             *
*  Retour      : Registre intégré à la liste manipulée.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArmV7Register *g_armv7_reglist_operand_get_register(const GArmV7RegListOperand *operand, size_t index)
{
    assert(index < operand->count);

    return operand->registers[index];

}
